# -*- coding: utf-8 -*-
# Generated by Django 1.11.26 on 2020-04-05 15:01
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('pymess', '0017_migration'),
    ]

    operations = [
        migrations.AddField(
            model_name='dialermessage',
            name='retry_sending',
            field=models.BooleanField(default=False, verbose_name='retry sending'),
        ),
        migrations.AddField(
            model_name='emailmessage',
            name='retry_sending',
            field=models.BooleanField(default=False, verbose_name='retry sending'),
        ),
        migrations.AddField(
            model_name='outputsmsmessage',
            name='retry_sending',
            field=models.BooleanField(default=False, verbose_name='retry sending'),
        ),
        migrations.AddField(
            model_name='pushnotificationmessage',
            name='retry_sending',
            field=models.BooleanField(default=False, verbose_name='retry sending'),
        ),
    ]
